@@ -3,21 +3,25 @@
 #
 # Function: Format PDF Output from groff Markup
 #
+# ------------------------------------------------------------------------------
+#
+# $Id: pdfroff.sh,v 1f246746f221 2025/04/23 19:05:45 keith $
+#
 # Copyright (C) 2005-2023 Free Software Foundation, Inc.
 # Written by Keith Marshall (keith.d.marshall@ntlworld.com)
-# 
+#
 # This file is part of groff.
-# 
+#
 # groff is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
 # Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # groff is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or
 # FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 # for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
@@ -215,8 +219,8 @@
 # Collect all other parameters into new argument and file lists,
 # to be passed on to 'groff', enforcing the '-Tps' option.
 #
-  DIFF="" STREAM="" INPUT_FILES=""
   SHOW_VERSION="" GROFF_STYLE="$GROFF -Tps -mpdfroff"
+  DIFF="" STREAM="" INPUT_FILES="" MANPDF_OPTIONS=false
   while test $# -gt 0
   do
     case "$1" in
@@ -391,7 +395,7 @@
       - | -i*)
 	   STREAM="$CAT ${GROFF_TMPDIR}/pdf$$.in |"
 	   test "$1" = "-" && INPUT_FILES="$INPUT_FILES $1" \
-	     || GROFF_STYLE="$GROFF_STYLE $1" 
+	     || GROFF_STYLE="$GROFF_STYLE $1"
 	   ;;
 #
 #     Those standard options which expect an argument, but are specified with
@@ -433,10 +437,13 @@
            ;;
 #
 #     All other standard options are simply passed through to 'groff',
-#     with no validation beforehand.
+#     with no validation beforehand; additionally, in the special case
+#     of the '-manpdf' macro package selection, we need to schedule an
+#     input stream prescan, to activate preprocessor sequences.
 #
       -*)  GROFF_STYLE="$GROFF_STYLE \"$1\""
-           ;;
+	   case "$1" in -manpdf) MANPDF_OPTIONS=true ;; esac
+	   ;;
 #
 #     All non-option arguments are considered as possible input file names,
 #     and are passed on to 'groff', unaltered.
@@ -527,6 +534,70 @@
     fi
   fi
 #
+# If the '-manpdf' macro package has been nominated to control formatting,
+# then we need to check the input stream for hints regarding preprocessors,
+# on which correctly formatted output may depend; reconstruct the sequence
+# of groff command line options, to activate any required preprocessors.
+#
+  ${MANPDF_OPTIONS-false} && {
+#
+#   According to man(7), preprocessor hints may be incorporated into
+#   any man page source, by placing a record of the form:
+#
+#       '\" <key-letters>
+#
+#   at the start of the page; valid key-letters are specified, in the
+#   nominal order in which groff will run them, as:
+#
+#     r  -  run the refer(1) preprocessor   (groff option: -R)
+#     g  -  run the grap(1) preprocessor    (groff option: -G)
+#     p  -  run the pic(1) preprocessor     (groff option: -p)
+#     t  -  run the tbl(1) preprocessor     (groff option: -t)
+#     e  -  run the eqn(1) preprocessor     (groff option: -e)
+#
+#   and, (unsupported by groff; excluded from <key-letter> matching):
+#
+#     v  -  run the vgrind(1) preprocessor
+#
+#   The following awk script will examine the entire groff input data
+#   stream, isolate any preprocessor <key-letter> records, (which MUST
+#   be placed BEFORE the '.TH', or '.Dd' macro call in each file), and
+#   construct a sequence of options, in groff's nominal running order,
+#   which will be added to the GROFF_STYLE command definition, thus
+#   requesting the use of the nominated preprocessors.
+#
+    MANPDF_OPTIONS(){ eval $STREAM $AWK -v optkeys="rgpte" "'
+	/^\\. *Dd/{ nextfile } /^\\. *TH/{ nextfile }
+	/^'\''\\"\\\""/{
+	  for( idx = 4; length() >= idx; idx++ )
+	    if( index( optkeys, c = substr( \$0, idx, 1 )) ) opt[c] = c
+	  nextfile
+	}
+	END {
+	  for( idx = 1; length( optkeys ) >= idx; idx++ )
+	    if( opt[c = substr( optkeys, idx, 1 )] ){
+	      if( idx < 3 ) c = toupper( c )
+	      optstr = optstr ? optstr c : \" -\"c
+	    }
+	  print optstr
+	}
+      '" '"$@"'
+    }
+    MANPDF_OPTIONS=`eval MANPDF_OPTIONS "$INPUT_FILES"`
+    test -n "$MANPDF_OPTIONS" && {
+#
+#     When preprocessor selection hints have been identified, rebuild
+#     the GROFF_STYLE command definition, to include the options.
+#
+      set -- $GROFF_STYLE
+      GROFF_STYLE="$1$MANPDF_OPTIONS"
+      while test $# -gt 1
+      do GROFF_STYLE="$GROFF_STYLE \"$2\""
+	 shift
+      done
+    }
+  }
+#
 # Prepare to run the multi-pass 'pdfmark' reference resolver loop ...
 #
 # Note that evaluation of in-document processing option hints, within this
@@ -588,7 +659,10 @@
 #   output at this stage.
 #
     eval $STREAM $GROFF_STYLE -Z 1>$NULLDEV 2>$WRKFILE $REFCOPY $INPUT_FILES
-    $AWK '/^gropdf-info:href/ {$1 = ".pdfhref D -N"; print}' $WRKFILE > $REFFILE
+    $AWK '/^gropdf-info:href/ { switch ($1)
+      { case /-keydef$/: $1 = "K"; break; default: $1 = "D -N"
+      }	print ".pdfhref "$0
+    }' $WRKFILE > $REFFILE
     if test "$PASS_INDICATOR" = "."
     then
 #
@@ -765,4 +839,4 @@
   $SAY >&2 ". done"
 #
 # ------------------------------------------------------------------------------
-# $RCSfile$ $Revision$: end of file
+# $RCSfile: pdfroff.sh,v $: end of file
